import numpy as np

def func(y_new, y_old, t_new, h):
    """ The function g(x) we want the root of."""
    K = 83.4772 
    r = 0.977361
    return y_new - y_old - h * (r * y_new - r / K * y_new ** 2)

def dfuncdx(y_new, h):
    """ The derivative of g(x) with respect to x_new."""
    K = 83.4772 
    r = 0.977361
    return 1 - h * (r - r / K * 2 * y_new)


def newtonforRK(y_old, t_new, h, TOL=1e-8):
    
    # Initial guess:
    y_new = y_old
    f = func(y_new, y_old, t_new, h)
    dfdx = dfuncdx(y_new, h)
    
    while abs(f/dfdx) > TOL:
        y_new = y_new - f/dfdx
        f = func(y_new, y_old, t_new, h)
        dfdx = dfuncdx(y_new, h)
        
    return y_new

def impliciteuler(y_0, t_0, t_f, N):
    h = (t_f - t_0) / N
    y = np.zeros((N+1))
    t = np.linspace(t_0, t_f, len(y))
    y[0] = y_0
    
    
    for n in range(N):
        y[n+1] = newtonforRK(y[n], t[n+1], h)    

    return h, t, y